/**
 * @class merchant
 * @extends Ext.Controller
 * The merchant controller
 */

/* 
   Known subclasses
      Ext.NearbyController
      Ext.CategoriesController
      Ext.FavoritesController
      Ext.SearchController
      Ext.CouponsController
*/
Ext.MerchantController = Ext.extend(Object, {

    list: function() {
        
    },

  // This is called from subclass of merchantController, e.g, nearby.js	
  /*
  	merchantObj contains information on the currently selected merchant. Information includes:
  	   merchant:     data about merchant, e.g., busname. See app/models/Merchant.js
       couponsStore: coupons offered by the merchant
     	 coupon:       the currently selected coupon
     	 animation:    transition information
       back:         stack for back buttons (who gets called when the back button is tapped.)
       favStore:     favorites shown when favorites tab is tapped. (stored in local database)
       data:         merchant data from test/responder.php
       
  */
	couponsList: function(merchantObj){
		this.action = 'couponsList';
		
		var currentView = this.application.viewport.getActiveItem();
		
		if(merchantObj.couponStore)
			this.couponsStore = merchantObj.couponStore;
		else 
			this.couponsStore = new Ext.data.Store({
				model:'Coupon',
				autoLoad:true,
				proxy:{
					type:'ajax',
					url:host+'CouponListResponder.php',
					extraParams:{
						sUUID:UUID,
						Latitude:lat,
						Longitude:lng,
						locID:merchantObj.merchant.locid,
					},
					reader:{type:'json'},
				}
			});
		
		this.merchantCouponsList = new Band.views.MerchantCouponsList({
			itemId:'merchant-coupons-panel',
			merchant:merchantObj.merchant,
			couponsStore:this.couponsStore,
			controller:this.id
		});
		
		this.merchantCouponsList.couponsList.on(
		             'itemtap',
		             function(list,index,item) {
				            merchantObj.couponsList=list.store;
				            merchantObj.coupon = list.store.getAt(index).data;
				            merchantObj.animation={type:'slide',direction:'left'};
				            merchantObj.back.push({controller:this.id, action:'couponsList'});
				
				           this.showCoupon(merchantObj);
				        },
				        this 
		);
		
		this.merchantCouponsList.on(
		      'activate',
		      function(item) {
             item.merchantPanel.el.on(
                'tap',
                function(){
                  merchantObj.couponsList=this.couponsStore;
                  merchantObj.animation={type:'slide',direction:'left'};
                  merchantObj.back.push({controller:this.id, action:'couponsList'});
                  this.details(merchantObj);
                },
                this
             )
          },
          this
		);
		
		this.merchantCouponsList.query('#back-merchant-coupons-'+this.id)[0].on(
		    'tap', 
		    function(item) {
			     var c = merchantObj.back.splice(merchantObj.back.length-1,1)[0];
			     Ext.apply(merchantObj, c);
			     merchantObj.animation = {type:'slide',direction:'right'};
			     Ext.dispatch(merchantObj);
		    },
		    this
		);
		
		if(!merchantObj.favStore) {
       merchantObj.favStore = new Ext.data.Store({model:'favorites'});
    }
    
		merchantObj.favStore.load( {
			scope:this, 
			callback:function(records) {
				this.toggleFavoritesIcon(merchantObj, this.merchantCouponsList);
		  }
		});
		
		
		currentView.add(this.merchantCouponsList);
		currentView.doLayout();
		currentView.setActiveItem('merchant-coupons-panel', merchantObj.animation);
	},
	
	toggleFavoritesIcon: function(merchantObj, o)
	{
		var fav = o.query('#fav')[0];
		if(merchantObj.favStore.findExact('busId',merchantObj.merchant.locid)>-1 ) {
			fav.action = 'remove';
			fav.setIcon('resources/images/FavoriteDel.png');
		}
		else {
			fav.action = 'add';
			fav.setIcon('resources/images/FavoriteAdd.png')
		}
	},

  details: function(merchantObj) {
		this.detailsPanel = new Band.views.MerchantView({
			itemId:'merchant-details-panel',
			merchant:merchantObj.merchant,
			controller:this.id
		});
		
		var currentView = this.application.viewport.getActiveItem();
		currentView.add(this.detailsPanel);
		currentView.doLayout();
		currentView.setActiveItem('merchant-details-panel',merchantObj.animation);
		this.detailsPanel.setLoading(true,true);
		
		var merchant = merchantObj.merchant;
		var This =this;
		if(!merchantObj.data){
			Ext.Ajax.request({
				url:host2+'test/responder.php',
				params:{
					sUUID:UUID,
					Latitude:lat,
					Longitude:lng,
					table:'tblBusiness',
					action:'BusDir_'+merchantObj.merchant.locid
				},
				method:'POST',
				success:function(response,options){
					merchantObj.data = Ext.util.JSON.decode( response.responseText);
					merchantObj.data.subject=encodeURIComponent('I saved at '+merchant.busname+' using Mobile Bandit');
					merchantObj.data.body=encodeURIComponent('I just saved at '+merchant.busname+' using The Mobile Bandit App for the iPhone.'+
								'\n\nThe best money saving App available.\n\nGet yours at www.iwantbandit.com\n\n\n'+
								merchant.busname+' is located at:\n'+merchant.address+'\nPhone: '+merchantObj.data.phone+'\n'+merchantObj.data.surl
								);
					Ext.apply(merchantObj.data,merchantObj.merchant);
					This.detailsPanel.update(merchantObj.data);
				}
			});
		}
        else {
			This.detailsPanel.update(merchantObj.data);
		}
		
		This.detailsPanel.body.on('tap',function(){
				This.showDirections(merchantObj);
			},
			this,{delegate:'.directions'});
		
		
		this.detailsPanel.query('#back-merchant-'+this.id)[0].on('tap', function(item){console.log(merchantObj.back);
			var c = merchantObj.back.splice(merchantObj.back.length-1,1)[0];
			console.log(c);
			Ext.apply(merchantObj,c);
			merchantObj.animation = {type:'slide',direction:'right'};
			Ext.dispatch(merchantObj);
		},this);
		if(!merchantObj.favStore) 
			merchantObj.favStore = new Ext.data.Store({model:'favorites'});
		merchantObj.favStore.load({scope:this, callback:function(records){
			if(merchantObj.favStore.findExact('busId',merchantObj.merchant.locid)>-1 ){
				this.detailsPanel.query('#fav')[0].action = 'remove';
				this.detailsPanel.query('#fav')[0].setIcon('resources/images/FavoriteDel.png');
			}else{
				this.detailsPanel.query('#fav')[0].action = 'add';
				this.detailsPanel.query('#fav')[0].setIcon('resources/images/FavoriteAdd.png')
			}
		}});
		
    },

	showCoupon:function(merchantObj){
		console.log('Coupon controler>> Show action');
		console.log(merchantObj);
		var currentView = this.application.viewport.getActiveItem();
		
		this.couponView = new Band.views.CouponView({
			itemId:'coupon-view-panel',
			merchant:merchantObj.merchant,
			coupon:merchantObj.coupon,
			data:Ext.apply(merchantObj.merchant, merchantObj.coupon),
			controller:this.id
		});

		this.couponView.doLayout();
		currentView.add(this.couponView);
		currentView.doLayout();
		currentView.setActiveItem('coupon-view-panel',merchantObj.animation);

		this.couponView.body.on('tap',function(item){
			if(localStorage.getItem('ActiveCouponID')){
				setTimeout(Band.checkForOverwrite(merchantObj),0);					// merchantObj is retained by closure
			}else{
				setTimeout(Band.setAndUpdateActiveCoupon(merchantObj),0);		// merchantObj is retained by closure 
			}
		},this,{delegate:'.btn-steal-it'});
		
		this.couponView.query('#back-coupon-'+this.id)[0].on('tap', function(item){
			var c = merchantObj.back.splice(merchantObj.back.length-1,1);
			Ext.apply(merchantObj,c);
			merchantObj.animation={type:'slide', direction:'right'};
			//Ext.dispatch(merchantObj);
			this.couponsList(merchantObj);
		},this);
		if(!merchantObj.favStore) 
			merchantObj.favStore = new Ext.data.Store({model:'favorites'});
		merchantObj.favStore.load({scope:this, callback:function(records){
			if(merchantObj.favStore.findExact('busId',merchantObj.merchant.locid)>-1 ){
				this.couponView.query('#fav')[0].action = 'remove';
				this.couponView.query('#fav')[0].setIcon('resources/images/FavoriteDel.png');
			}else{
				this.couponView.query('#fav')[0].action = 'add';
				this.couponView.query('#fav')[0].setIcon('resources/images/FavoriteAdd.png')
			}
		}});
		
		Ext.Ajax.request({
			url:host+'responder.php',
			params:{
//'table':'tblCoupon','action':actionString,'LID':iSecLoc,'Latitude':thisLat,'Longitude':thisLong,'sUUID':thisUUID
				sUUID:UUID,
				Latitude:lat,
				Longitude:lng,
				table:'tblCoupon',
				action:'CouponInfo_' + merchantObj.coupon.id
			},
			method:'POST',
			callback:function(option,success,response){
				data = Ext.util.JSON.decode(response.responseText);
				console.log(data);
				if (data.lastchance == 1) {
			        //TimerExpireText.text = 'YOUR LAST CHANCE TO STEAL THIS DEAL!';
			        //TimerExpireText.color = '#33ccff';

			        if(!localStorage.getItem('dontShowLastChance')){
			            var MB = new Ext.MessageBox;
						Ext.apply(MB, {
								NO: { text: 'OK', itemId: 'no', ui: 'action' },
								YES: { text: 'Don\'t show', itemId: 'yes' }
						});
						Ext.apply(MB, {
								YESNO: [MB.NO, MB.YES]
						});

						MB.confirm('DONT LET THIS ONE GET AWAY!','This is your last chance to steal this deal. You let the last one expire before using it.', 
							function(e){
								if(e == 'yes'){
									localStorage.setItem('dontShowLastChance', true);
								}
							}
						);

			        }
				}
			}		
		});				
	},				// end of showCoupon method
	
	
	
	showDirections:function(merchantObj){
		var directionMap = new Band.views.DirectionView({
			itemId:'directions-map',
			origin:lat+','+lng,
			destination:merchantObj.data.lat+','+merchantObj.data.lng,
			merchant:merchantObj.merchant,
			controller:this.id
		});
		
		directionMap.query('#back-directions-'+this.id)[0].on('tap', function(item){
			merchantObj.animation={type:'slide', direction:'right'};
			this.details(merchantObj);
		},this);
		
		var currentView = this.application.viewport.getActiveItem();
		currentView.add(directionMap);
		currentView.doLayout();
		currentView.setActiveItem('directions-map',{type:'slide',direction:'left'});
	},	
});



