/**
 * @class nearby
 * @extends Ext.Controller
 * The nearby controller
 */
/*global Ext, Band, host, UUID, lat, lng, console*/  /* Used by JSLint*/
(function () {     // this wrapper is used so that JSLint does not complain about 'use strict'
    'use strict';
    Ext.NearbyController = Ext.extend(Ext.MerchantController, {    // Ext.MerchantController 
        // The first 3 actions are invoked by clicking buttons on the top bar displayed on the nearby tab
        nearest: function (controller) {
            this.createMerchantList('nearest', controller, this.getMerchantData);
        },

        newest: function (controller) {
            this.createMerchantList('newest', controller, this.getMerchantData);
        },

        tophits: function (controller) {
            this.createMerchantList('tophits', controller, this.getMerchantData);
        },

        // Displays merchants ordered by listType 
        // Note: this should be moved to Ext.MerchantsList so that it can be used by the favorites and search controllers
        // The getData method would be different for the favorites and search controllers.
        createMerchantList: function (listType, controller, getData) {
            var store,
                storeName = listType + '-store',
                listName = listType + '-list';
                
            if (this.application.viewport.getActiveItem() !== this.application.viewport.getComponent('nearby-panel')) {
                this.application.viewport.setActiveItem('nearby-panel', {
                    type: 'slide',
                    direction: 'right'
                });
            }	
            this.action = listType;
            if (!this.nearbyIndexPanel) {
                this.nearbyIndexPanel = this.application.viewport.getComponent('nearby-panel').getComponent('nearby-index');
            }

            store = this[storeName];		// Each list type has a different store
            if (!store) {
                store = this[storeName] = getData.call(this, listType);			
            } else if (store.getCount() === 0) {
                store.load();
            }
    
            // if list has not already been created, do so now and save it.
            if (!this[listName]) {
                this[listName] = new Band.views.MerchantList({
                    itemId: listType,
                    store: store,
                    order: listType,
                    listeners: {
                        scope: this,
                        deactivate: function (item) {
                            console.log('destroying ' + listType + 'list');
                            item.destroy();
                            this[listName] = null;
                        },
                        itemtap: function (controller, index, item) {
                            item = null;     // to silence JSLint, could also use arguments
                            this.couponsList({			// in "super class", see app/controllers/merchant.js
                                merchant: controller.store.getAt(index).data,
                                back: [{
                                    controller: 'nearby',
                                    action: listType
                                }],
                                animation: {
                                    type: 'slide',
                                    direction: 'left'
                                }
                            });
                        }
                    }
                });

                // implement continuous scrolling
                this[listName].scroller.on('scrollend', function (scroller, offset) {
                    scroller = null;				// to silence JSLint, could also use arguments.
                    this.pageSize = 25;// 25 is the page size
                    this.itemHeight = 80;//80 pixel is the fixed height of each item
                    var oy = offset.y,
                        itemsCount = oy / this.itemHeight, //how many items have been scrolled (disapeared)
                        showenItems = this.ownerCt.getHeight() / this.itemHeight;//how many Items are shown

                    //start loading the next 25 record whenn the user views half
                    if (store.getCount() - this.pageSize / 2 <= itemsCount + showenItems && !store.isLoading() && !this.isLastPage) {
                        this.loadingText = '';//to disable the loading mask and give the use infinit scrolling
                        this.loadMask.disable();
                        this.isLastPage = (this.store.getCount() % this.pageSize === 0 ? false : true);//supposing that if we got 25 records it means there are more!!
                        this.prevRowsCount = this.store.getCount();
                        store.nextPage();
                    }
                }, this[listName]);
						
                this.nearbyIndexPanel.add(this[listName]);
                this.nearbyIndexPanel.doLayout();
                this.nearbyIndexPanel.setActiveItem(listType);			
            }
				
            this.application.viewport.getComponent('nearby-panel').setActiveItem('nearby-index', (controller.animation || null));
            this.application.viewport.getComponent('nearby-panel').doLayout();
            Ext.getCmp('nearby-tabbar').setPressed(listType + '-button', true);        
        },
        
        
        // returns data that is used to display a list of Merchants.
        // -listtype:string (value: nearest, newest, tophits) determines how the server sorts the data, 
        getMerchantData: function (listType) {
            return new Band.NearbyStore({
                model: 'Merchant',    // in app/models/Merchant.js
                autoLoad: true,
                proxy: {
                    type: 'ajax',
                    url: host + 'NearbyResponder.php',
					
                    extraParams: {
                        sUUID: UUID,
                        Latitude: lat,
                        Longitude: lng,
                        offset: 0,
                        order: listType    // determines how server sorts data.
                    },

                    reader: {
                        type:  'json',
                        root:  '0',
                        total: 1
                    }
                }
            });
        }
    });

    Ext.regController("nearby", new Ext.NearbyController()); 

    /*
	The default pagin for sencha is to send a calculated start and limit params each time the user requsted new page
	in the mobile badit app the pagin is handled by 'offset' param instead of start and the limit is hardcoded in the server to 25
	So we need to override the loadPage of the Store to modify our offset parametre
*/
    Band.NearbyStore = Ext.extend(Ext.data.Store, {
	
        constructor: function (config) {
	    
            Band.NearbyStore.superclass.constructor.call(this, config);
        // Band.NearbyStore.superclass.initComponent.apply(this, arguments);
        },
	
        loadPage: function (page) {
            this.currentPage = page;
		
            this.proxy.extraParams.offset = (page - 1) * this.pageSize;
		
            this.read({
                page : page,
                start: (page - 1) * this.pageSize,
                limit: this.pageSize,
                addRecords: true
            });
        }
    });
}());