// F. Permadi 2005.
// (C) F. Permadi
// http://www.permadi.com/tutorial/domTree/index.html
// Print DOM tree
////////////////////////////////////////////
// This function traverses the DOM tree of an element and prints the tree.  
// This function called recursively until the DOM tree is fully traversed.
// 
// Parameters:
// - targetDocument is where the tree will be printed into
// - currentElement is the element that we want to print
// - depth is the depth of the current element 
//   (it should be 1 for the initial element)
////////////////////////////////////////////
function traverseDOMTree(targetDocument, currentElement, depth)
{
  if (currentElement)
  {
    var j;
    var tagName=currentElement.tagName;
    // Prints the node tagName, such as <a>, <img>, etc
    if (currentElement.nodeType == 3)
    	targetDocument.writeln(currentElement.nodeValue);
    else if (tagName)
    {
    	tagName = tagName.toLowerCase();    	
    	for (var i=0; i<depth; i++) {
    		targetDocument.write("....");
    	}
      targetDocument.writeln("&lt;"+tagName);
      if (currentElement.attributes)
      {
      	var len = currentElement.attributes.length;
      	for (var i=0; i<len; i++) {
      		var att = currentElement.attributes[i];
      		targetDocument.write(" " + att.nodeName + "='" + att.nodeValue +"'");
      	}
      }
      if (tagName === "br" || tagName === "input" || tagName === "img" || tagName === "script") {
      	targetDocument.write("/><br/>");
      	return;					// assume no children
      }
      else {
      	targetDocument.write("&gt;<br/>");
    	}
    }
    else if (currentElement.nodeType == 8) {
    	return // ignore comment
    }
    else {
      targetDocument.write("nodeType=" + currentElement.nodeType);		// should never reach here
      return;
    }

    // Traverse the tree
    var i=0;
    var currentElementChild=currentElement.childNodes[i];
    while (currentElementChild)
    {
      // Recursively traverse the tree structure of the child node
      traverseDOMTree(targetDocument, currentElementChild, depth+1);
      i++;
      currentElementChild=currentElement.childNodes[i];
    }
    
    if (currentElement.nodeType != 3) {
    	for (var i = 0; i<depth; i++) {
    		targetDocument.write("&nbsp;");
    	}
    	
    	targetDocument.write("&lt;/" + tagName + "&gt;<br/>");
    }
  }
}

////////////////////////////////////////////
// This function accepts a DOM element as parameter and prints
// out the DOM tree structure of the element.
////////////////////////////////////////////
function printDOMTree(domElement, destinationWindow)
{
  // Use destination window to print the tree.  If destinationWIndow is
  //   not specified, create a new window and print the tree into that window
  var outputWindow=destinationWindow;
  if (!outputWindow)
    outputWindow=window.open();

  // make a valid html page
  outputWindow.document.open("text/html", "replace");
  outputWindow.document.write("<html><head><title>DOM</title></head><body>\n");
  outputWindow.document.write("<code>\n");
  traverseDOMTree(outputWindow.document, domElement, 1);
  outputWindow.document.write("</code>\n");
  outputWindow.document.write("</body></html>\n");
  
  // Here we must close the document object, otherwise Mozilla browsers 
  //   might keep showing "loading in progress" state.
  outputWindow.document.close();
}
